@@ -116,6 +116,9 @@ class Generate_Logger:
     # Class methods that can be accessed using the class instance.
     def log_info(self, msg):
        self.logger.info(msg)
+       
+    def log_warning(self, msg):
+        self.logger.warning(msg)        
     
     def log_error(self, msg):
         self.logger.error(msg)
@@ -133,17 +136,34 @@ class Generate_Logger:
 
 
 #################################################################
-# Templates for database connections.
+# Templates for database connections
 # Depending on the type of connection, proper key word arguments 
 # must be passed to get the connection variable.
 #################################################################
 
 def get_kds_teradata_conn(**kwargs):
-    udaExec = teradata.UdaExec (appName='kds', version="2", configureLogging=False, logConsole = False)
-    conn = udaExec.connect(DSN=kwargs.get('ODBC_name'),method="odbc")
+    
+    trycntr = 1
+    stoptrying = False
+    
+    while not stoptrying:
+
+        try:   
+            udaExec = teradata.UdaExec (appName='kds', version="2", configureLogging=False, logConsole = False)
+            conn = udaExec.connect(DSN=kwargs.get('ODBC_name'),method="odbc")
+            stoptrying = True
+        except Exception as e:            
+            if trycntr >= 3:
+                print('Unable to connect to EDW: ' + str(e))
+                conn = e
+                stoptrying = True
+                
+            trycntr += 1
+            time.sleep(3)
 
     return conn
 
+
 def get_sqlalchemy_teradata_engine(**kwargs):
 
     if  kwargs.get('username') is not None:
@@ -159,6 +179,7 @@ def get_sqlalchemy_teradata_engine(**kwargs):
 
     return edw_engine
 
+
 def get_pyodbc_windowsauth_conn(**kwargs):
    
     params_str = urllib.quote("""Driver={driver};Server={server};Database={database};Trusted_Connection=yes;""".format(driver=kwargs.get('driver'),
@@ -168,6 +189,7 @@ def get_pyodbc_windowsauth_conn(**kwargs):
     
     return conn_str
 
+
 def get_pyodbc_windowsauth_sa_engine(**kwargs):
    
     params_str = urllib.quote("""Driver={driver};Server={server};Database={database};Trusted_Connection=yes;""".format(driver=kwargs.get('driver'),
@@ -179,6 +201,7 @@ def get_pyodbc_windowsauth_sa_engine(**kwargs):
     
     return sqlsaengine
 
+
 def get_pyodbc_windowsauth_cnxn(**kwargs):
 
     params_str = ("Driver={driver};Server={server};Database={database};Trusted_Connection={trusted_connection};").format(
@@ -191,6 +214,7 @@ def get_pyodbc_windowsauth_cnxn(**kwargs):
     
     return conn    
 
+
 def get_pyodbc_credentials_conn(**kwargs):
     
     if  kwargs.get('username') == None:
@@ -209,6 +233,7 @@ def get_pyodbc_credentials_conn(**kwargs):
     
     return conn
 
+
 def get_pyodbc_credentials_sa_engine(**kwargs):
     
     if  kwargs.get('username') == None:
@@ -231,6 +256,7 @@ def get_pyodbc_credentials_sa_engine(**kwargs):
     
     return sqlACEngine
 
+
 def get_pyodbc_azure_ad_auth_conn(**kwargs):
     params_str = ("Driver={driver};Server={server};Database={database};Authentication={auth};").format(
         driver=kwargs.get('driver'),
@@ -241,6 +267,7 @@ def get_pyodbc_azure_ad_auth_conn(**kwargs):
     conn = pyodbc.connect(params_str)
     
     return conn
+
     
 def get_pyodbc_azure_credentials_conn(**kwargs):
     
@@ -260,6 +287,7 @@ def get_pyodbc_azure_credentials_conn(**kwargs):
     
     return conn   
 
+
 def get_sqlalchemy_sql_credentials_engine(**kwargs):
 
     if  kwargs.get('username') is not None:
@@ -275,6 +303,7 @@ def get_sqlalchemy_sql_credentials_engine(**kwargs):
                                                                                                 password=quote_plus(kwargs.get('password')))).connect()    
 
     return sql_engine
+
     
 def senderroremail(**kwargs):
     
@@ -290,6 +319,9 @@ def senderroremail(**kwargs):
     s.send_message(msg)
     s.quit()
     
+    return
+
+    
 def emaillogfile(**kwargs):
     
     programName_ = kwargs.get('programName')
@@ -309,6 +341,9 @@ def emaillogfile(**kwargs):
     s = smtplib.SMTP(cfg.get('EMAIL').get('SMTP'))
     s.send_message(msg)
     s.quit()
+    
+    return
+
 
 def sendemailnotification(**kwargs):
     
@@ -331,21 +366,30 @@ def sendemailnotification(**kwargs):
     s.send_message(msg)
     s.quit()
     
-def read_parquet_files(path):
-    listOfParquetFiles = glob.glob(path + "/*.parquet")
+    return
+
+
+def read_parquet_files(path, match_pattern=None):    
+    listOfParquetFiles = glob.glob(path + "/*.parquet") if match_pattern is None else glob.glob(path + f"/*{match_pattern}*.parquet")
     df = [pd.read_parquet(f) for f in listOfParquetFiles]
     df_all = pd.concat(df,ignore_index=True)
     df_all.reset_index(inplace=True, drop=True)
+
     return df_all
 
+
 def create_folder(folder, DeleteIfExists):
     if DeleteIfExists:
         if Path(folder).is_dir():
             shutil.rmtree(folder)
-    if not os.path.exists(folder):
-        os.makedirs(folder)
+            os.makedirs(folder)
+    else:
+        if not os.path.exists(folder):
+            os.makedirs(folder)
+        
     return
 
+
 def generate_metadata(*args):
     
     file = args[0]
